@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: alsa_driver.c,v 1.68 2005/05/10 03:21:18 pbd Exp $
+    $Id: alsa_driver.c,v 1.70 2005/08/22 20:48:24 rncbc Exp $
 */
 
 #include <math.h>
@@ -1681,7 +1681,10 @@ alsa_driver_run_cycle (alsa_driver_t *driver)
 
 	DEBUG ("alsaback from wait, nframes = %lu", nframes);
 
-	if (nframes == 0) {
+	if (unlikely(wait_status < 0))
+		return -1;		/* driver failed */
+
+	if (unlikely(nframes == 0)) {
 
 		/* we detected an xrun and restarted: notify
 		 * clients about the delay. 
@@ -1690,13 +1693,7 @@ alsa_driver_run_cycle (alsa_driver_t *driver)
 		return 0;
 	} 
 
-	if (wait_status == 0)
-		return engine->run_cycle (engine, nframes, delayed_usecs);
-
-	if (wait_status < 0)
-		return -1;		/* driver failed */
-	else
-		return 0;
+	return engine->run_cycle (engine, nframes, delayed_usecs);
 }
 
 static int
