@@ -15,15 +15,14 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: alsa_driver.h,v 1.5 2004/03/25 19:31:50 joq Exp $
+    $Id: alsa_driver.h,v 1.11 2005/03/16 21:19:31 essej Exp $
 */
 
 #ifndef __jack_alsa_driver_h__
 #define __jack_alsa_driver_h__
 
-#define ALSA_PCM_OLD_HW_PARAMS_API
-#define ALSA_PCM_OLD_SW_PARAMS_API
 #include <alsa/asoundlib.h>
+#include <jack/bitset.h>
 
 #if __BYTE_ORDER == __LITTLE_ENDIAN
 	#define	SND_PCM_FORMAT_S24_3 SND_PCM_FORMAT_S24_3LE
@@ -51,7 +50,7 @@ typedef void (*CopyCopyFunction)  (char *dst, char *src,
 				   unsigned long dst_skip_bytes,
 				   unsigned long src_skip_byte);
 
-typedef struct {
+typedef struct _alsa_driver {
 
     JACK_DRIVER_NT_DECL
 
@@ -84,14 +83,14 @@ typedef struct {
     char                         *alsa_name_playback;
     char                         *alsa_name_capture;
     char                         *alsa_driver;
-    snd_pcm_uframes_t             buffer_frames;
-    unsigned long                 channels_not_done;
-    unsigned long                 channel_done_bits;
+    bitset_t			  channels_not_done;
+    bitset_t			  channels_done;
     snd_pcm_format_t              playback_sample_format;
     snd_pcm_format_t              capture_sample_format;
     float                         max_sample_val;
     unsigned long                 user_nperiods;
-    unsigned long                 nfragments;
+    unsigned int                  playback_nperiods;
+    unsigned int                  capture_nperiods;
     unsigned long                 last_mask;
     snd_ctl_t                    *ctl_handle;
     snd_pcm_t                    *playback_handle;
@@ -105,6 +104,7 @@ typedef struct {
     jack_client_t                *client;
     JSList                       *capture_ports;
     JSList                       *playback_ports;
+    JSList                       *monitor_ports;
 
     unsigned long input_monitor_mask;
 
@@ -135,17 +135,21 @@ typedef struct {
     int running;
     int run;
 
+    int poll_late;
     int xrun_count;
     int process_count;
 
 } alsa_driver_t;
 
-static __inline__ void alsa_driver_mark_channel_done (alsa_driver_t *driver, channel_t chn) {
-	driver->channels_not_done &= ~(1<<chn);
+static inline void 
+alsa_driver_mark_channel_done (alsa_driver_t *driver, channel_t chn) {
+	bitset_remove (driver->channels_not_done, chn);
 	driver->silent[chn] = 0;
 }
 
-static __inline__ void alsa_driver_silence_on_channel (alsa_driver_t *driver, channel_t chn, jack_nframes_t nframes) {
+static inline void 
+alsa_driver_silence_on_channel (alsa_driver_t *driver, channel_t chn,
+				jack_nframes_t nframes) {
 	if (driver->playback_interleaved) {
 		memset_interleave 
 			(driver->playback_addr[chn],
@@ -153,12 +157,15 @@ static __inline__ void alsa_driver_silence_on_channel (alsa_driver_t *driver, ch
 			 driver->interleave_unit,
 			 driver->playback_interleave_skip);
 	} else {
-		memset (driver->playback_addr[chn], 0, nframes * driver->playback_sample_bytes);
+		memset (driver->playback_addr[chn], 0,
+			nframes * driver->playback_sample_bytes);
 	}
 	alsa_driver_mark_channel_done (driver,chn);
 }
 
-static __inline__ void alsa_driver_silence_on_channel_no_mark (alsa_driver_t *driver, channel_t chn, jack_nframes_t nframes) {
+static inline void 
+alsa_driver_silence_on_channel_no_mark (alsa_driver_t *driver, channel_t chn,
+					jack_nframes_t nframes) {
 	if (driver->playback_interleaved) {
 		memset_interleave 
 			(driver->playback_addr[chn],
@@ -166,13 +173,16 @@ static __inline__ void alsa_driver_silence_on_channel_no_mark (alsa_driver_t *dr
 			 driver->interleave_unit,
 			 driver->playback_interleave_skip);
 	} else {
-		memset (driver->playback_addr[chn], 0, nframes * driver->playback_sample_bytes);
+		memset (driver->playback_addr[chn], 0,
+			nframes * driver->playback_sample_bytes);
 	}
 }
 
-static __inline__ void alsa_driver_read_from_channel (alsa_driver_t *driver, 
-					   channel_t channel, jack_default_audio_sample_t *buf, 
-					   jack_nframes_t nsamples)
+static inline void 
+alsa_driver_read_from_channel (alsa_driver_t *driver,
+			       channel_t channel,
+			       jack_default_audio_sample_t *buf,
+			       jack_nframes_t nsamples)
 {
 	driver->read_via_copy (buf, 
 			       driver->capture_addr[channel],
@@ -180,10 +190,11 @@ static __inline__ void alsa_driver_read_from_channel (alsa_driver_t *driver,
 			       driver->capture_interleave_skip);
 }
 
-static __inline__ void alsa_driver_write_to_channel (alsa_driver_t *driver,
-				   channel_t channel, 
-				   jack_default_audio_sample_t *buf, 
-				   jack_nframes_t nsamples)
+static inline void 
+alsa_driver_write_to_channel (alsa_driver_t *driver,
+			      channel_t channel, 
+			      jack_default_audio_sample_t *buf, 
+			      jack_nframes_t nsamples)
 {
 	driver->write_via_copy (driver->playback_addr[channel],
 				buf, 
@@ -193,10 +204,11 @@ static __inline__ void alsa_driver_write_to_channel (alsa_driver_t *driver,
 	alsa_driver_mark_channel_done (driver, channel);
 }
 
-static __inline__ void alsa_driver_copy_channel (alsa_driver_t *driver, 
-			       channel_t input_channel, 
-			       channel_t output_channel,
-			       jack_nframes_t nsamples) {
+static inline void 
+alsa_driver_copy_channel (alsa_driver_t *driver, 
+			  channel_t input_channel, 
+			  channel_t output_channel,
+			  jack_nframes_t nsamples) {
 
 	driver->channel_copy (driver->playback_addr[output_channel],
 			      driver->capture_addr[input_channel],
@@ -206,10 +218,16 @@ static __inline__ void alsa_driver_copy_channel (alsa_driver_t *driver,
 	alsa_driver_mark_channel_done (driver, output_channel);
 }
 
-void  alsa_driver_set_clock_sync_status (alsa_driver_t *driver, channel_t chn, ClockSyncStatus status);
-int   alsa_driver_listen_for_clock_sync_status (alsa_driver_t *, ClockSyncListenerFunction, void *arg);
-int   alsa_driver_stop_listen_for_clock_sync_status (alsa_driver_t *, unsigned int);
-void  alsa_driver_clock_sync_notify (alsa_driver_t *, channel_t chn, ClockSyncStatus);
-
+void  alsa_driver_silence_untouched_channels (alsa_driver_t *driver,
+					      jack_nframes_t nframes);
+void  alsa_driver_set_clock_sync_status (alsa_driver_t *driver, channel_t chn,
+					 ClockSyncStatus status);
+int   alsa_driver_listen_for_clock_sync_status (alsa_driver_t *,
+						ClockSyncListenerFunction,
+						void *arg);
+int   alsa_driver_stop_listen_for_clock_sync_status (alsa_driver_t *,
+						     unsigned int);
+void  alsa_driver_clock_sync_notify (alsa_driver_t *, channel_t chn,
+				     ClockSyncStatus);
 
 #endif /* __jack_alsa_driver_h__ */
