@@ -1,6 +1,6 @@
 /*
     Copyright (C) 2001 Paul Davis 
-    Copyright (C) 2004 Karsten Wiese, Rui Nuno Capela
+    Copyright (C) 2005 Karsten Wiese, Rui Nuno Capela
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: usx2y.c,v 1.3 2005/06/09 22:10:35 rncbc Exp $
+    $Id: usx2y.c,v 1.5 2005/10/30 11:01:31 rncbc Exp $
 */
 
 #include <jack/hardware.h>
@@ -31,7 +31,12 @@
 #define ARRAY_SIZE(x) (sizeof(x) / sizeof((x)[0]))
 #endif
 
+//#define DBGHWDEP
 
+#ifdef DBGHWDEP
+int dbg_offset;
+char dbg_buffer[8096];
+#endif
 static 
 int usx2y_set_input_monitor_mask (jack_hardware_t *hw, unsigned long mask)
 {
@@ -118,7 +123,7 @@ usx2y_driver_get_channel_addresses_playback (alsa_driver_t *driver,
 	if (dbg_offset < (sizeof(dbg_buffer) - 256))
 		dbg_offset += sprintf(dbg_buffer + dbg_offset, "avail %li@%p\n", *playback_avail, driver->playback_addr[0]);
 	else {
-		jack_error(dbg_buffer);
+		printf(dbg_buffer);
 		return -1;
 	}
 #endif
@@ -143,7 +148,7 @@ usx2y_driver_get_channel_addresses_capture (alsa_driver_t *driver,
 			return 0; /* FIXME: return -1; */
 		h->capture_iso_bytes_done = 0;
 #ifdef DBGHWDEP
-		dbg_offset = sprintf(dbg_buffer, "first iso = %i %i@%p:%i\n",
+		dbg_offset = sprintf(dbg_buffer, "cfirst iso = %i %i@%p:%i\n",
 					iso, h->hwdep_pcm_shm->captured_iso[iso].length,
 					h->hwdep_pcm_shm->capture0x8,
 					h->hwdep_pcm_shm->captured_iso[iso].offset);
@@ -152,7 +157,7 @@ usx2y_driver_get_channel_addresses_capture (alsa_driver_t *driver,
 		iso = h->capture_iso_start;
 	}
 #ifdef DBGHWDEP
-	dbg_offset += sprintf(dbg_buffer + dbg_offset, "iso = %i(%i;%i); ", iso,
+	dbg_offset += sprintf(dbg_buffer + dbg_offset, "ciso = %i(%i;%i); ", iso,
 				h->hwdep_pcm_shm->captured_iso[iso].offset,
 				h->hwdep_pcm_shm->captured_iso[iso].frame);
 #endif
@@ -178,10 +183,21 @@ usx2y_driver_get_channel_addresses_capture (alsa_driver_t *driver,
 			((chn & 1) ? driver->capture_sample_bytes : 0);
 	}
 #ifdef DBGHWDEP
+ {
+	int f = 0;
+	unsigned *u = driver->capture_addr[0];
+	static unsigned last;
+	dbg_offset += sprintf(dbg_buffer + dbg_offset, "\nvon %6u  bis %6u\n", last, u[0]);
+	while (f < *capture_avail && dbg_offset < (sizeof(dbg_buffer) - 256)) {
+		if (u[f] != last + 1)
+			 dbg_offset += sprintf(dbg_buffer + dbg_offset, "\nooops %6u  %6u\n", last, u[f]);
+		last = u[f++];
+	}
+ }
 	if (dbg_offset < (sizeof(dbg_buffer) - 256))
 		dbg_offset += sprintf(dbg_buffer + dbg_offset, "avail %li@%p\n", *capture_avail, driver->capture_addr[0]);
 	else {
-		jack_error(dbg_buffer);
+		printf(dbg_buffer);
 		return -1;
 	}
 #endif
@@ -197,6 +213,12 @@ usx2y_driver_start (alsa_driver_t *driver)
 
 	usx2y_t *h = (usx2y_t *) driver->hw->private;
 
+	if (driver->capture_nchannels == 4) {
+		// US428 channels 3+4 are on a seperate 2 channel stream.
+		// ALSA thinks its 1 stream with 4 channels, so we have to hack here.
+		driver->capture_interleave_skip = 2 * driver->capture_sample_bytes;
+	}
+
 	driver->poll_last = 0;
 	driver->poll_next = 0;
 
@@ -443,9 +465,9 @@ usx2y_driver_null_cycle (alsa_driver_t* driver, jack_nframes_t nframes)
 static int
 usx2y_driver_read (alsa_driver_t *driver, jack_nframes_t nframes)
 {
-	snd_pcm_sframes_t contiguous;
+	snd_pcm_uframes_t contiguous;
 	snd_pcm_sframes_t nread;
-	snd_pcm_sframes_t offset;
+	snd_pcm_uframes_t offset;
 	jack_default_audio_sample_t* buf[4];
 	channel_t chn;
 	JSList *node;
@@ -458,61 +480,55 @@ usx2y_driver_read (alsa_driver_t *driver, jack_nframes_t nframes)
 	}
 
 	nread = 0;
-	contiguous = 0;
+
+	if (snd_pcm_mmap_begin (driver->capture_handle,
+				&driver->capture_areas,
+				&offset, &nframes_) < 0) {
+		jack_error ("ALSA/USX2Y: %s: mmap areas info error",
+			    driver->alsa_name_capture);
+		return -1;
+	}
+
+	for (chn = 0, node = driver->capture_ports;
+	     node; node = jack_slist_next (node), chn++) {
+		port = (jack_port_t *) node->data;
+		if (!jack_port_connected (port)) {
+			continue;
+		}
+		buf[chn] = jack_port_get_buffer (port, nframes_);
+	}
 
 	while (nframes) {
 
-		contiguous = (nframes > driver->frames_per_cycle) ?
-			driver->frames_per_cycle : nframes;
-
-		if (snd_pcm_mmap_begin (
-				driver->capture_handle, &driver->capture_areas,
-				(snd_pcm_uframes_t *) &offset,
-				(snd_pcm_uframes_t *) &nframes_) < 0) {
-			jack_error ("ALSA/USX2Y: %s: mmap areas info error",
-				driver->alsa_name_capture);
+		contiguous = nframes;
+		if (usx2y_driver_get_channel_addresses_capture (
+			    driver, &contiguous) < 0) {
 			return -1;
 		}
-
 		for (chn = 0, node = driver->capture_ports;
-				node && chn < 4;
-					node = jack_slist_next (node), chn++) {
+		     node; node = jack_slist_next (node), chn++) {
 			port = (jack_port_t *) node->data;
 			if (!jack_port_connected (port)) {
+				/* no-copy optimization */
 				continue;
 			}
-			buf[chn] = jack_port_get_buffer (port, nframes_);
+			alsa_driver_read_from_channel (driver, chn,
+						       buf[chn] + nread,
+						       contiguous);
+/* 			sample_move_dS_s24(buf[chn] + nread, */
+/* 					   driver->capture_addr[chn], */
+/* 					   contiguous, */
+/* 					   driver->capture_interleave_skip); */
 		}
+		nread += contiguous;
+		nframes -= contiguous;
+	}
 
-		while (nframes) {
-			contiguous = nframes;
-			if (usx2y_driver_get_channel_addresses_capture (
-					driver, &contiguous) < 0) {
-				return -1;
-			}
-			for (chn = 0, node = driver->capture_ports;
-					node && chn < 4;
-						node = jack_slist_next (node), chn++) {
-				port = (jack_port_t *) node->data;
-				if (!jack_port_connected (port)) {
-					/* no-copy optimization */
-					continue;
-				}
-				alsa_driver_read_from_channel (driver, chn,
-					buf[chn] + nread, contiguous);
-			}
-			nread += contiguous;
-			nframes -= contiguous;
-		}
-
-		if ((err = snd_pcm_mmap_commit (driver->capture_handle,
-				offset, nframes_)) < 0) {
-			jack_error ("ALSA/USX2Y: could not complete read of %"
-				PRIu32 " frames: error = %d", nframes_, err);
-			return -1;
-		}
-
-//		nframes -= contiguous;
+	if ((err = snd_pcm_mmap_commit (driver->capture_handle,
+					offset, nframes_)) < 0) {
+		jack_error ("ALSA/USX2Y: could not complete read of %"
+			    PRIu32 " frames: error = %d", nframes_, err);
+		return -1;
 	}
 
 	return 0;
@@ -525,11 +541,10 @@ usx2y_driver_write (alsa_driver_t* driver, jack_nframes_t nframes)
 	JSList *node;
 	jack_default_audio_sample_t* buf[2];
 	snd_pcm_sframes_t nwritten;
-	snd_pcm_sframes_t contiguous;
-	snd_pcm_sframes_t offset;
+	snd_pcm_uframes_t contiguous;
+	snd_pcm_uframes_t offset;
 	jack_port_t *port;
 	int err;
-	int dbg_loops = 1;
 	snd_pcm_uframes_t nframes_ = nframes;
 
 	driver->process_count++;
@@ -539,7 +554,6 @@ usx2y_driver_write (alsa_driver_t* driver, jack_nframes_t nframes)
 	}
 
 	nwritten = 0;
-	contiguous = 0;
 
 	/* check current input monitor request status */
 
@@ -561,83 +575,45 @@ usx2y_driver_write (alsa_driver_t* driver, jack_nframes_t nframes)
 		}
 	}
 
-	while (nframes) {
-
-		if (--dbg_loops)
-			return dbg_loops;
-
-		contiguous = (nframes > driver->frames_per_cycle) ?
-			driver->frames_per_cycle : nframes;
-
-		if (snd_pcm_mmap_begin(
-				driver->playback_handle, &driver->playback_areas,
-				&offset, &nframes_) < 0) {
-			jack_error ("ALSA/USX2Y: %s: mmap areas info error",
-				driver->alsa_name_capture);
-			return -1;
-		}
-
-		for (chn = 0, node = driver->playback_ports;
-				node && chn < 2;
-					node = jack_slist_next (node), chn++) {
-			port = (jack_port_t *) node->data;
-			if (!jack_port_connected (port)) {
-				continue;
-			}
-			buf[chn] = jack_port_get_buffer (port, nframes_);
-		}
-
-		while (nframes) {
-			contiguous = nframes;
-			if (usx2y_driver_get_channel_addresses_playback (
-					driver, &contiguous) < 0) {
-				return -1;
-			}
-			for (chn = 0, node = driver->playback_ports;
-					node && chn < 2;
-						node = jack_slist_next (node), chn++) {
-				port = (jack_port_t *) node->data;
-				if (!jack_port_connected (port)) {
-					continue;
-				}
-				alsa_driver_write_to_channel (driver, chn,
-					buf[chn] + nwritten,
-					contiguous);
-			}
-			nwritten += contiguous;
-			if (driver->channels_not_done) {
-				alsa_driver_silence_untouched_channels (driver,
-					contiguous);
-			}
-			nframes -= contiguous;
-		}
-
-		if ((err = snd_pcm_mmap_commit (driver->playback_handle,
-				offset, nframes_)) < 0) {
-			jack_error ("ALSA/USX2Y: could not complete playback of %"
-				PRIu32 " frames: error = %d", nframes_, err);
-			if (err != EPIPE && err != ESTRPIPE)
-				return -1;
-		}
-//		nframes -= contiguous;
+	if (snd_pcm_mmap_begin(driver->playback_handle,
+			       &driver->playback_areas,
+			       &offset, &nframes_) < 0) {
+		jack_error ("ALSA/USX2Y: %s: mmap areas info error",
+			    driver->alsa_name_capture);
+		return -1;
 	}
 
-/* 	{ */
-/*      usx2y_t *h = (usx2y_t *) driver->hw->private; */
-/* 		unsigned *pu = (unsigned *)h->hwdep_pcm_shm->playback; */
-/* 		int i = sizeof(h->hwdep_pcm_shm->playback) / sizeof(*pu); */
+	for (chn = 0, node = driver->playback_ports;
+	     node; node = jack_slist_next (node), chn++) {
+		port = (jack_port_t *) node->data;
+		buf[chn] = jack_port_get_buffer (port, nframes_);
+	}
 
-/* 		while (i) { */
-/* 			if (*(pu)) { */
-/* 				jack_error("%p;error %u(=0x%X)@%p", */
-/* 					   h->hwdep_pcm_shm->playback, */
-/* 					   *pu, *pu, pu); */
-/* 				return -1; */
-/* 			} */
-/* 			--i; */
-/* 			++pu; */
-/* 		} */
-/* 	} */
+	while (nframes) {
+
+		contiguous = nframes;
+		if (usx2y_driver_get_channel_addresses_playback (
+			    driver, &contiguous) < 0) {
+			return -1;
+		}
+		for (chn = 0, node = driver->playback_ports;
+		     node; node = jack_slist_next (node), chn++) {
+			port = (jack_port_t *) node->data;
+			alsa_driver_write_to_channel (driver, chn,
+						      buf[chn] + nwritten,
+						      contiguous);
+		}
+		nwritten += contiguous;
+		nframes -= contiguous;
+	}
+
+	if ((err = snd_pcm_mmap_commit (driver->playback_handle,
+					offset, nframes_)) < 0) {
+		jack_error ("ALSA/USX2Y: could not complete playback of %"
+			    PRIu32 " frames: error = %d", nframes_, err);
+		if (err != EPIPE && err != ESTRPIPE)
+			return -1;
+	}
 
 	return 0;
 }
