@@ -1,7 +1,7 @@
 /*
 
 	OSS driver for Jack
-	Copyright (C) 2003-2004 Jussi Laako <jussi@sonarnerd.net>
+	Copyright (C) 2003-2005 Jussi Laako <jussi@sonarnerd.net>
 
 	This program is free software; you can redistribute it and/or modify
 	it under the terms of the GNU General Public License as published by
@@ -33,6 +33,12 @@
  */
 #define _XOPEN_SOURCE	600
 #endif
+#ifndef _REENTRANT
+#define _REENTRANT
+#endif
+#ifndef _THREAD_SAFE
+#define _THREAD_SAFE
+#endif
 #include <unistd.h>
 #include <pthread.h>
 #include <stdlib.h>
@@ -60,7 +66,7 @@
 #include "oss_driver.h"
 
 
-#define OSS_DRIVER_N_PARAMS	9
+#define OSS_DRIVER_N_PARAMS	11
 const static jack_driver_param_desc_t oss_params[OSS_DRIVER_N_PARAMS] = {
 	{ "rate",
 	  'r',
@@ -124,6 +130,20 @@ const static jack_driver_param_desc_t oss_params[OSS_DRIVER_N_PARAMS] = {
 	  { },
 	  "ignore hardware period size",
 	  "ignore hardware period size"
+	},
+	{ "input latency",
+	  'I',
+	  JackDriverParamUInt,
+	  { .ui = 0 },
+	  "system input latency",
+	  "system input latency"
+	},
+	{ "output latency",
+	  'O',
+	  JackDriverParamUInt,
+	  { .ui = 0 },
+	  "system output latency",
+	  "system output latency"
 	}
 };
 
@@ -243,8 +263,10 @@ static void copy_and_convert_out (void *dst, jack_sample_t *src,
 			scale = 0x7fff;
 			for (srcidx = 0; srcidx < nframes; srcidx++)
 			{
-				s16dst[dstidx] = (signed short) 
-					(src[srcidx] * scale + 0.5f);
+				s16dst[dstidx] = (signed short)
+					(src[srcidx] >= 0.0f) ?
+					(src[srcidx] * scale + 0.5f) :
+					(src[srcidx] * scale - 0.5f);
 				dstidx += chcount;
 			}
 			break;
@@ -253,7 +275,9 @@ static void copy_and_convert_out (void *dst, jack_sample_t *src,
 			for (srcidx = 0; srcidx < nframes; srcidx++)
 			{
 				s32dst[dstidx] = (signed int)
-					(src[srcidx] * scale + 0.5f);
+					(src[srcidx] >= 0.0f) ?
+					(src[srcidx] * scale + 0.5f) :
+					(src[srcidx] * scale - 0.5f);
 				dstidx += chcount;
 			}
 			break;
@@ -262,7 +286,9 @@ static void copy_and_convert_out (void *dst, jack_sample_t *src,
 			for (srcidx = 0; srcidx < nframes; srcidx++)
 			{
 				s32dst[dstidx] = (signed int)
-					(src[srcidx] * scale + 0.5f);
+					(src[srcidx] >= 0.0f) ?
+					(src[srcidx] * scale + 0.5f) :
+					(src[srcidx] * scale - 0.5f);
 				dstidx += chcount;
 			}
 			break;
@@ -286,8 +312,8 @@ static void set_fragment (int fd, size_t fragsize, unsigned int fragcount)
 	fragments = ((fragcount << 16) | (fragsize_2p & 0xffff));
 	if (ioctl(fd, SNDCTL_DSP_SETFRAGMENT, &fragments) < 0)
 	{
-		jack_error("OSS: failed to set fragment size: %s@%i",
-			__FILE__, __LINE__);
+		jack_error("OSS: failed to set fragment size: %s@%i, errno=%d",
+			__FILE__, __LINE__, errno);
 	}
 }
 
@@ -298,8 +324,8 @@ static int get_fragment (int fd)
 
 	if (ioctl(fd, SNDCTL_DSP_GETBLKSIZE, &fragsize) < 0)
 	{
-		jack_error("OSS: failed to get fragment size: %s@%i",
-			__FILE__, __LINE__);
+		jack_error("OSS: failed to get fragment size: %s@%i, errno=%d",
+			__FILE__, __LINE__, errno);
 		return 0;
 	}
 	return fragsize;
@@ -337,6 +363,8 @@ static int oss_driver_attach (oss_driver_t *driver, jack_engine_t *engine)
 				channel_name, __FILE__, __LINE__);
 			break;
 		}
+		jack_port_set_latency(port,
+			driver->period_size + driver->sys_in_latency);
 		driver->capture_ports = 
 			jack_slist_append(driver->capture_ports, port);
 	}
@@ -354,6 +382,8 @@ static int oss_driver_attach (oss_driver_t *driver, jack_engine_t *engine)
 				channel_name, __FILE__, __LINE__);
 			break;
 		}
+		jack_port_set_latency(port,
+			driver->period_size + driver->sys_out_latency);
 		driver->playback_ports =
 			jack_slist_append(driver->playback_ports, port);
 	}
@@ -434,8 +464,8 @@ static int oss_driver_start (oss_driver_t *driver)
 			if (infd < 0)
 			{
 				jack_error(
-					"OSS: failed to open input device %s: %s@%i",
-					indev, __FILE__, __LINE__);
+					"OSS: failed to open input device %s: %s@%i, errno=%d",
+					indev, __FILE__, __LINE__, errno);
 			}
 			fragsize = driver->period_size * 
 				driver->capture_channels * samplesize;
@@ -449,8 +479,8 @@ static int oss_driver_start (oss_driver_t *driver)
 			if (outfd < 0)
 			{
 				jack_error(
-					"OSS: failed to open output device %s: %s@%i",
-					outdev, __FILE__, __LINE__);
+					"OSS: failed to open output device %s: %s@%i, errno=%d",
+					outdev, __FILE__, __LINE__, errno);
 			}
 			fragsize = driver->period_size * 
 				driver->playback_channels * samplesize;
@@ -468,8 +498,8 @@ static int oss_driver_start (oss_driver_t *driver)
 			if (infd < 0)
 			{
 				jack_error(
-					"OSS: failed to open device %s: %s@%i",
-					indev, __FILE__, __LINE__);
+					"OSS: failed to open device %s: %s@%i, errno=%d",
+					indev, __FILE__, __LINE__, errno);
 				return -1;
 			}
 		}
@@ -481,8 +511,8 @@ static int oss_driver_start (oss_driver_t *driver)
 			if (outfd < 0)
 			{
 				jack_error(
-					"OSS: failed to open device %s: %s@%i",
-					outdev, __FILE__, __LINE__);
+					"OSS: failed to open device %s: %s@%i, errno=%d",
+					outdev, __FILE__, __LINE__, errno);
 				return -1;
 			}
 		}
@@ -492,8 +522,8 @@ static int oss_driver_start (oss_driver_t *driver)
 			if (infd < 0)
 			{
 				jack_error(
-					"OSS: failed to open device %s: %s@%i",
-					indev, __FILE__, __LINE__);
+					"OSS: failed to open device %s: %s@%i, errno=%d",
+					indev, __FILE__, __LINE__, errno);
 				return -1;
 			}
 		}
@@ -501,9 +531,11 @@ static int oss_driver_start (oss_driver_t *driver)
 		{
 			if (ioctl(infd, SNDCTL_DSP_SETDUPLEX, 0) < 0)
 			{
-				jack_error(
-					"OSS: failed to enable full duplex for %s: %s@%i",
-					indev, __FILE__, __LINE__);
+				if (errno != EINVAL) /* Dont care */
+					jack_error(
+						"OSS: failed to enable full duplex for %s: %s@%i, errno=%d",
+						indev, __FILE__, __LINE__,
+						errno);
 			}
 		}
 		if (infd >= 0)
@@ -527,18 +559,18 @@ static int oss_driver_start (oss_driver_t *driver)
 		format = driver->format;
 		if (ioctl(infd, SNDCTL_DSP_SETFMT, &format) < 0)
 			jack_error(
-				"OSS: failed to set format for %s: %s@%i", 
-				indev, __FILE__, __LINE__);
+				"OSS: failed to set format for %s: %s@%i, errno=%d", 
+				indev, __FILE__, __LINE__, errno);
 		channels = driver->capture_channels;
 		if (ioctl(infd, SNDCTL_DSP_CHANNELS, &channels) < 0)
 			jack_error(
-				"OSS: failed to set channels for %s: %s@%i", 
-				indev, __FILE__, __LINE__);
+				"OSS: failed to set channels for %s: %s@%i, errno=%d", 
+				indev, __FILE__, __LINE__, errno);
 		samplerate = driver->sample_rate;
 		if (ioctl(infd, SNDCTL_DSP_SPEED, &samplerate) < 0)
 			jack_error(
-				"OSS: failed to set samplerate for %s: %s@%i", 
-				indev, __FILE__, __LINE__);
+				"OSS: failed to set samplerate for %s: %s@%i, errno=%d", 
+				indev, __FILE__, __LINE__, errno);
 		printf("oss_driver: %s : 0x%x/%i/%i (%i)\n", indev, 
 			format, channels, samplerate, get_fragment(infd));
 		
@@ -562,18 +594,18 @@ static int oss_driver_start (oss_driver_t *driver)
 		format = driver->format;
 		if (ioctl(outfd, SNDCTL_DSP_SETFMT, &format) < 0)
 			jack_error(
-				"OSS: failed to set format for %s: %s@%i", 
-				outdev, __FILE__, __LINE__);
+				"OSS: failed to set format for %s: %s@%i, errno=%d", 
+				outdev, __FILE__, __LINE__, errno);
 		channels = driver->playback_channels;
 		if (ioctl(outfd, SNDCTL_DSP_CHANNELS, &channels) < 0)
 			jack_error(
-				"OSS: failed to set channels for %s: %s@%i", 
-				outdev, __FILE__, __LINE__);
+				"OSS: failed to set channels for %s: %s@%i, errno=%d", 
+				outdev, __FILE__, __LINE__, errno);
 		samplerate = driver->sample_rate;
 		if (ioctl(outfd, SNDCTL_DSP_SPEED, &samplerate) < 0)
 			jack_error(
-				"OSS: failed to set samplerate for %s: %s@%i", 
-				outdev, __FILE__, __LINE__);
+				"OSS: failed to set samplerate for %s: %s@%i, errno=%d", 
+				outdev, __FILE__, __LINE__, errno);
 		printf("oss_driver: %s : 0x%x/%i/%i (%i)\n", outdev, 
 			format, channels, samplerate, 
 			get_fragment(outfd));
@@ -637,19 +669,22 @@ static int oss_driver_start (oss_driver_t *driver)
 	pthread_mutex_init(&driver->mutex_in, NULL);
 	pthread_mutex_init(&driver->mutex_out, NULL);
 #	ifdef USE_BARRIER
+	puts("oss_driver: using barrier mode, (dual thread)");
 	pthread_barrier_init(&driver->barrier, NULL, 2);
+#	else
+	puts("oss_driver: not using barrier mode, (single thread)");
 #	endif
 	sem_init(&driver->sem_start, 0, 0);
 	driver->run = 1;
 	driver->threads = 0;
 	if (infd >= 0)
 	{
-		if (jack_create_thread(&driver->thread_in, 
+		if (jack_client_create_thread(NULL, &driver->thread_in, 
 			driver->engine->rtpriority, 
 			driver->engine->control->real_time, 
 			io_thread, driver) < 0)
 		{
-			jack_error("OSS: jack_create_thread() failed: %s@%i",
+			jack_error("OSS: jack_client_create_thread() failed: %s@%i",
 				__FILE__, __LINE__);
 			return -1;
 		}
@@ -658,12 +693,12 @@ static int oss_driver_start (oss_driver_t *driver)
 #	ifdef USE_BARRIER
 	if (outfd >= 0)
 	{
-		if (jack_create_thread(&driver->thread_out, 
+		if (jack_client_create_thread(NULL, &driver->thread_out, 
 			driver->engine->rtpriority, 
 			driver->engine->control->real_time, 
 			io_thread, driver) < 0)
 		{
-			jack_error("OSS: jack_create_thread() failed: %s@%i",
+			jack_error("OSS: jack_client_create_thread() failed: %s@%i",
 				__FILE__, __LINE__);
 			return -1;
 		}
@@ -877,6 +912,7 @@ static inline void synchronize (oss_driver_t *driver)
 static void *io_thread (void *param)
 {
 	size_t localsize;
+	ssize_t io_res;
 	void *localbuf;
 	oss_driver_t *driver = (oss_driver_t *) param;
 
@@ -896,11 +932,13 @@ static void *io_thread (void *param)
 
 		while (driver->run)
 		{
-			if (read(driver->infd, localbuf, localsize) <
-				(ssize_t) localsize)
+			io_res = read(driver->infd, localbuf, localsize);
+			if (io_res < (ssize_t) localsize)
 			{
-				jack_error("OSS: read() failed: %s@%i", 
-					__FILE__, __LINE__);
+				jack_error(
+					"OSS: read() failed: %s@%i, count=%d/%d, errno=%d",
+					__FILE__, __LINE__, io_res, localsize,
+					errno);
 				break;
 			}
 
@@ -930,11 +968,13 @@ static void *io_thread (void *param)
 			memcpy(localbuf, driver->outdevbuf, localsize);
 			pthread_mutex_unlock(&driver->mutex_out);
 
-			if (write(driver->outfd, localbuf, localsize) < 
-				(ssize_t) localsize)
+			io_res = write(driver->outfd, localbuf, localsize);
+			if (io_res < (ssize_t) localsize)
 			{
-				jack_error("OSS: write() failed: %s@%i", 
-					__FILE__, __LINE__);
+				jack_error(
+					"OSS: write() failed: %s@%i, count=%d/%d, errno=%d",
+					__FILE__, __LINE__, io_res, localsize,
+					errno);
 				break;
 			}
 
@@ -962,24 +1002,28 @@ static void *io_thread (void *param)
 				driver->outdevbufsize);
 			pthread_mutex_unlock(&driver->mutex_out);
 
-			if (write(driver->outfd, localbuf, 
-				driver->outdevbufsize) <
-				(ssize_t) driver->outdevbufsize)
+			io_res = write(driver->outfd, localbuf, 
+				driver->outdevbufsize);
+			if (io_res < (ssize_t) driver->outdevbufsize)
 			{
-				jack_error("OSS: write() failed: %s@%i",
-					__FILE__, __LINE__);
+				jack_error(
+					"OSS: write() failed: %s@%i, count=%d/%d, errno=%d",
+					__FILE__, __LINE__, io_res,
+					driver->outdevbufsize, errno);
 				break;
 			}
 		}
 
 		if (driver->capture_channels > 0)
 		{
-			if (read(driver->infd, localbuf, 
-				driver->indevbufsize) <
-				(ssize_t) driver->indevbufsize)
+			io_res = read(driver->infd, localbuf, 
+				driver->indevbufsize);
+			if (io_res < (ssize_t) driver->indevbufsize)
 			{
-				jack_error("OSS: read() failed: %s@%i",
-					__FILE__, __LINE__);
+				jack_error(
+					"OSS: read() failed: %s@%i, count=%d/%d, errno=%d",
+					__FILE__, __LINE__, io_res,
+					driver->indevbufsize, errno);
 				break;
 			}
 
@@ -1016,16 +1060,22 @@ jack_driver_desc_t * driver_get_descriptor ()
 	desc = (jack_driver_desc_t *) calloc(1, sizeof(jack_driver_desc_t));
 	if (desc == NULL)
 	{
-		printf("oss_driver: malloc() failed: %s@%i\n", 
-			__FILE__, __LINE__);
+		printf("oss_driver: calloc() failed: %s@%i, errno=%d\n",
+			__FILE__, __LINE__, errno);
 		return NULL;
 	}
 	strcpy(desc->name, driver_client_name);
 	desc->nparams = OSS_DRIVER_N_PARAMS;
 
 	params = calloc(desc->nparams, sizeof(jack_driver_param_desc_t));
+	if (params == NULL)
+	{
+		printf("oss_driver: calloc() failed: %s@%i, errno=%d\n",
+			__FILE__, __LINE__, errno);
+		return NULL;
+	}
 	memcpy(params, oss_params, 
-		OSS_DRIVER_N_PARAMS * sizeof(jack_driver_param_desc_t));
+		desc->nparams * sizeof(jack_driver_param_desc_t));
 	desc->params = params;
 
 	return desc;
@@ -1038,6 +1088,8 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 	int bits = OSS_DRIVER_DEF_BITS;
 	jack_nframes_t sample_rate = OSS_DRIVER_DEF_FS;
 	jack_nframes_t period_size = OSS_DRIVER_DEF_BLKSIZE;
+	jack_nframes_t in_latency = 0;
+	jack_nframes_t out_latency = 0;
 	unsigned int nperiods = OSS_DRIVER_DEF_NPERIODS;
 	unsigned int capture_channels = OSS_DRIVER_DEF_INS;
 	unsigned int playback_channels = OSS_DRIVER_DEF_OUTS;
@@ -1048,7 +1100,8 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 	driver = (oss_driver_t *) malloc(sizeof(oss_driver_t));
 	if (driver == NULL)
 	{
-		jack_error("OSS: malloc() failed: %s@%i", __FILE__, __LINE__);
+		jack_error("OSS: malloc() failed: %s@%i, errno=%d",
+			__FILE__, __LINE__, errno);
 		return NULL;
 	}
 	jack_driver_init((jack_driver_t *) driver);
@@ -1101,6 +1154,12 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 			case 'b':
 				driver->ignorehwbuf = 1;
 				break;
+			case 'I':
+				in_latency = param->value.ui;
+				break;
+			case 'O':
+				out_latency = param->value.ui;
+				break;
 		}
 		pnode = jack_slist_next(pnode);
 	}
@@ -1111,6 +1170,8 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 	driver->bits = bits;
 	driver->capture_channels = capture_channels;
 	driver->playback_channels = playback_channels;
+	driver->sys_in_latency = in_latency;
+	driver->sys_out_latency = out_latency;
 
 	set_period_size(driver, period_size);
 	
@@ -1189,8 +1250,13 @@ jack_driver_t * driver_initialize (jack_client_t *client,
 
 void driver_finish (jack_driver_t *driver)
 {
-	free(((oss_driver_t *) driver)->indev);
-	free(((oss_driver_t *) driver)->outdev);
+	oss_driver_t *oss_driver = (oss_driver_t *) driver;
+
+	oss_driver = (oss_driver_t *) driver;
+	if (oss_driver->indev != NULL)
+		free(oss_driver->indev);
+	if (oss_driver->outdev != NULL)
+		free(oss_driver->outdev);
 	free(driver);
 }
 
