@@ -19,7 +19,7 @@
     * 2002/08/23 - modify for libsndfile 1.0.0 <andy@alsaplayer.org>
     * 2003/05/26 - use ringbuffers - joq
     
-    $Id: capture_client.c,v 1.9 2003/10/31 20:52:23 joq Exp $
+    $Id: capture_client.c,v 1.11 2004/03/27 22:08:55 joq Exp $
 */
 
 #include <stdio.h>
@@ -45,7 +45,7 @@ typedef struct _thread_info {
     volatile int can_capture;
     volatile int can_process;
     volatile int status;
-} thread_info_t;
+} jack_thread_info_t;
 
 /* JACK data */
 unsigned int nports;
@@ -65,7 +65,7 @@ long overruns = 0;
 void *
 disk_thread (void *arg)
 {
-	thread_info_t *info = (thread_info_t *) arg;
+	jack_thread_info_t *info = (jack_thread_info_t *) arg;
 	static jack_nframes_t total_captured = 0;
 	jack_nframes_t samples_per_frame = info->channels;
 	size_t bytes_per_frame = samples_per_frame * sample_size;
@@ -116,7 +116,7 @@ process (jack_nframes_t nframes, void *arg)
 {
 	int chn;
 	size_t i;
-	thread_info_t *info = (thread_info_t *) arg;
+	jack_thread_info_t *info = (jack_thread_info_t *) arg;
 
 	/* Do nothing until we're ready to begin. */
 	if ((!info->can_process) || (!info->can_capture))
@@ -158,7 +158,7 @@ jack_shutdown (void *arg)
 }
 
 void
-setup_disk_thread (thread_info_t *info)
+setup_disk_thread (jack_thread_info_t *info)
 {
 	SF_INFO sf_info;
 	int short_mask;
@@ -195,7 +195,7 @@ setup_disk_thread (thread_info_t *info)
 }
 
 void
-run_disk_thread (thread_info_t *info)
+run_disk_thread (jack_thread_info_t *info)
 {
 	info->can_capture = 1;
 	pthread_join (info->thread_id, NULL);
@@ -213,7 +213,7 @@ run_disk_thread (thread_info_t *info)
 }
 
 void
-setup_ports (int sources, char *source_names[], thread_info_t *info)
+setup_ports (int sources, char *source_names[], jack_thread_info_t *info)
 {
 	unsigned int i;
 	size_t in_size;
@@ -261,7 +261,7 @@ main (int argc, char *argv[])
 
 {
 	jack_client_t *client;
-	thread_info_t thread_info;
+	jack_thread_info_t thread_info;
 	int c;
 	int longopt_index = 0;
 	extern int optind, opterr;
