@@ -38,7 +38,7 @@ jack_client_t *client;
  * The default time signature is: "march time", 4/4, 120bpm
  */
 float time_beats_per_bar = 4.0;
-float time_beat_type = 0.25;
+float time_beat_type = 4.0;
 double time_ticks_per_beat = 1920.0;
 double time_beats_per_minute = 120.0;
 volatile int time_reset = 1;		/* true when time values change */
@@ -422,6 +422,8 @@ void command_loop()
 
 int main(int argc, char *argv[])
 {
+	jack_status_t status;
+
 	/* basename $0 */
 	package = strrchr(argv[0], '/');
 	if (package == 0)
@@ -429,9 +431,11 @@ int main(int argc, char *argv[])
 	else
 		package++;
 
-	/* become a new client of the JACK server */
-	if ((client = jack_client_new(package)) == 0) {
-		fprintf(stderr, "jack server not running?\n");
+	/* open a connection to the JACK server */
+	client = jack_client_open (package, JackNullOption, &status);
+	if (client == NULL) {
+		fprintf (stderr, "jack_client_open() failed, "
+			 "status = 0x%2.0x\n", status);
 		return 1;
 	}
 
