@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: engine.h,v 1.44 2004/08/22 21:39:53 pbd Exp $
+    $Id: engine.h,v 1.50 2004/12/19 18:41:28 pbd Exp $
 */
 
 #ifndef __jack_engine_h__
@@ -26,9 +26,6 @@
 #include <jack/internal.h>
 #include <jack/driver_interface.h>
 
-#define VERBOSE(engine,format,args...) \
-	if ((engine)->verbose) fprintf (stderr, format, ## args)
-
 struct _jack_driver;
 struct _jack_client_internal;
 struct _jack_port_internal;
@@ -64,12 +61,12 @@ struct _jack_engine {
     jack_driver_desc_t    *driver_desc;
     JSList                *driver_params;
 
-    /* these are "callbacks" made by the driver */
+    /* these are "callbacks" made by the driver backend */
     int  (*set_buffer_size) (struct _jack_engine *, jack_nframes_t frames);
     int  (*set_sample_rate) (struct _jack_engine *, jack_nframes_t frames);
     int  (*run_cycle)	    (struct _jack_engine *, jack_nframes_t nframes,
 			     float delayed_usecs);
-    void (*delay)	    (struct _jack_engine *);
+    void (*delay)	    (struct _jack_engine *, float delayed_usecs);
     void (*transport_cycle_start) (struct _jack_engine *, jack_time_t time);
     void (*driver_exit)     (struct _jack_engine *);
 
@@ -113,9 +110,11 @@ struct _jack_engine {
     char	    freewheeling;
     char	    verbose;
     char	    do_munlock;
+    const char	   *server_name;
     char	    temporary;
     int		    reordered;
     int		    watchdog_check;
+    int		    feedbackcount;
     pid_t           wait_pid;
     pthread_t       freewheel_thread;
 
@@ -137,7 +136,8 @@ struct _jack_engine {
     int		    rolling_interval;
     float	    max_usecs;
     float	    spare_usecs;
-    float	    usecs_per_cycle;
+
+    int first_wakeup;
     
 #ifdef JACK_USE_MACH_THREADS
     /* specific resources for server/client real-time thread communication */
@@ -150,7 +150,8 @@ struct _jack_engine {
 /* public functions */
 
 jack_engine_t  *jack_engine_new (int real_time, int real_time_priority,
-				 int do_mlock, int do_unlock, int temporary,
+				 int do_mlock, int do_unlock,
+				 const char *server_name, int temporary,
 				 int verbose, int client_timeout,
 				 unsigned int port_max,
                                  pid_t waitpid, JSList *drivers);
@@ -162,6 +163,12 @@ int		jack_engine_load_driver (jack_engine_t *engine,
 					 JSList * driver_params);
 void		jack_dump_configuration(jack_engine_t *engine, int take_lock);
 
+/* private engine functions */
+void		jack_engine_reset_rolling_usecs (jack_engine_t *engine);
+int		internal_client_request (void* ptr, jack_request_t *request);
+int		jack_get_fifo_fd (jack_engine_t *engine,
+				  unsigned int which_fifo);
+
 extern jack_client_internal_t *
 jack_client_internal_by_id (jack_engine_t *engine, jack_client_id_t id);
 
@@ -187,4 +194,11 @@ static inline unsigned int jack_power_of_two (unsigned int n)
 	return !(n & (n - 1));
 }
 
+/* Internal port handling interfaces for JACK engine. */
+void	jack_port_clear_connections (jack_engine_t *engine,
+				     jack_port_internal_t *port);
+void	jack_port_registration_notify (jack_engine_t *, jack_port_id_t, int);
+void	jack_port_release (jack_engine_t *engine, jack_port_internal_t *);
+void	jack_sort_graph (jack_engine_t *engine);
+
 #endif /* __jack_engine_h__ */
