@@ -1,5 +1,6 @@
 /*
     Copyright (C) 2001 Paul Davis
+    Copyright (C) 2004 Jack O'Quin
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +16,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: jack.h,v 1.64 2004/07/15 03:07:28 trutkin Exp $
+    $Id: jack.h,v 1.75 2005/05/11 02:06:46 pbd Exp $
 */
 
 #ifndef __jack_h__
@@ -34,23 +35,63 @@ extern "C" {
  * Note: More documentation can be found in jack/types.h.
  */
 
+/**
+ * Open an external client session with a JACK server.  This interface
+ * is more complex but more powerful than jack_client_new().  With it,
+ * clients may choose which of several servers to connect, and control
+ * whether and how to start the server automatically, if it was not
+ * already running.  There is also an option for JACK to generate a
+ * unique client name, when necessary.
+ *
+ * @param client_name of at most jack_client_name_size() characters.
+ * The name scope is local to each server.  Unless forbidden by the
+ * @ref JackUseExactName option, the server will modify this name to
+ * create a unique variant, if needed.
+ *
+ * @param options formed by OR-ing together @ref JackOptions bits.
+ * Only the @ref JackOpenOptions bits are allowed.
+ *
+ * @param status (if non-NULL) an address for JACK to return
+ * information from the open operation.  This status word is formed by
+ * OR-ing together the relevant @ref JackStatus bits.
+ *
+ *
+ * <b>Optional parameters:</b> depending on corresponding [@a options
+ * bits] additional parameters may follow @a status (in this order).
+ *
+ * @arg [@ref JackServerName] <em>(char *) server_name</em> selects
+ * from among several possible concurrent server instances.  Server
+ * names are unique to each user.  If unspecified, use "default"
+ * unless \$JACK_DEFAULT_SERVER is defined in the process environment.
+ *
+ * @return Opaque client handle if successful.  If this is NULL, the
+ * open operation failed, @a *status includes @ref JackFailure and the
+ * caller is not a JACK client.
+ */
+jack_client_t *jack_client_open (const char *client_name,
+				 jack_options_t options,
+				 jack_status_t *status, ...);
+
 /**
  * Attempt to become an external client of the Jack server.
  *
  * JACK is evolving a mechanism for automatically starting the server
  * when needed.  As a transition, jack_client_new() only does this
- * when $JACK_START_SERVER is defined in the environment of the
+ * when \$JACK_START_SERVER is defined in the environment of the
  * calling process.  In the future this will become normal behavior.
- * In either case, defining $JACK_NO_START_SERVER disables this
+ * For full control of this feature, use jack_client_open(), instead.
+ * In either case, defining \$JACK_NO_START_SERVER disables this
  * feature.
  *
  * @param client_name of at most jack_client_name_size() characters.
+ * If this name is already in use, the request fails.
  *
  * @return Opaque client handle if successful, otherwise NULL.
  *
  * @note Failure generally means that the JACK server is not running.
  * If there was some other problem, it will be reported via the @ref
- * jack_error_callback mechanism.
+ * jack_error_callback mechanism.  Use jack_client_open() and check
+ * the @a status parameter for more detailed information.
  */
 jack_client_t *jack_client_new (const char *client_name);
 
@@ -65,30 +106,45 @@ int jack_client_close (jack_client_t *client);
  * @return the maximum number of characters in a JACK client name
  * including the final NULL character.  This value is a constant.
  */
-int jack_client_name_size(void);
+int jack_client_name_size (void);
 
 /**
- * Attempt to load an internal client into the Jack server.
+ * @return pointer to actual client name.  This is useful when @ref
+ * JackUseExactName is not specified on open and @ref
+ * JackNameNotUnique status was returned.  In that case, the actual
+ * name will differ from the @a client_name requested.
+ */
+char *jack_get_client_name (jack_client_t *client);
+
+/**
+ * Load an internal client into the Jack server.
  *
- * Internal clients run within the JACK server process.  They can use
- * of the same functions as external clients.  Each internal client
- * must declare jack_initialize() and jack_finish() entry points,
- * called at load and unload times.  See inprocess.c for an example of
- * how to write an internal client.
+ * Internal clients run inside the JACK server process.  They can use
+ * most of the same functions as external clients.  Each internal
+ * client must declare jack_initialize() and jack_finish() entry
+ * points, called at load and unload times.  See inprocess.c for an
+ * example of how to write an internal client.
+ *
+ * @deprecated Please use jack_internal_client_load().
  *
  * @param client_name of at most jack_client_name_size() characters.
- * @param so_name A path to a shared object file containing the code
- * for the new client.
- * @param so_data An arbitary string containing information to be
- * passed to the jack_initialize() routine of the new client.
+ *
+ * @param load_name of a shared object file containing the code for
+ * the new client.
+ *
+ * @param load_init an arbitary string passed to the jack_initialize()
+ * routine of the new client (may be NULL).
+ *
+ * @return 0 if successful.
  */
-int jack_internal_client_new (const char *client_name, const char *so_name,
-			      const char *so_data);
+int jack_internal_client_new (const char *client_name,
+			      const char *load_name,
+			      const char *load_init);
 
 /**
- * Removes an internal client from a JACK server.
+ * Remove an internal client from a JACK server.
  *
- * @return 0 on success, otherwise a non-zero error code
+ * @deprecated Please use jack_internal_client_load().
  */
 void jack_internal_client_close (const char *client_name);
 
@@ -119,7 +175,8 @@ int jack_is_realtime (jack_client_t *client);
  * to help more complex clients understand what is going
  * on.  It should be called before jack_client_activate().
  */
-void jack_on_shutdown (jack_client_t *client, void (*function)(void *arg), void *arg);
+void jack_on_shutdown (jack_client_t *client,
+		       void (*function)(void *arg), void *arg);
 
 /**
  * Tell the Jack server to call @a process_callback whenever there is
@@ -180,6 +237,12 @@ int jack_set_freewheel_callback (jack_client_t *client,
  * scheduling is dropped when entering freewheel mode, and
  * if appropriate it is reacquired when stopping.
  * 
+ * IMPORTANT: on systems using capabilities to provide real-time
+ * scheduling (i.e. Linux kernel 2.4), if onoff is zero, this function
+ * must be called from the thread that originally called jack_activate(). 
+ * This restriction does not apply to other systems (e.g. Linux kernel 2.6 
+ * or OS X).
+ * 
  * @param client pointer to JACK client structure
  * @param onoff  if non-zero, freewheel mode starts. Otherwise
  *                  freewheel mode ends.
@@ -232,8 +295,8 @@ int jack_set_sample_rate_callback (jack_client_t *client,
 				   void *arg);
 
 /**
- * Tell the Jack server to call 'registration_callback' whenever a port is registered
- * or unregistered, passing 'arg' as a second argument.
+ * Tell the JACK server to call @a registration_callback whenever a
+ * port is registered or unregistered, passing @a arg as a parameter.
  *
  * @return 0 on success, otherwise a non-zero error code
  */
@@ -242,20 +305,23 @@ int jack_set_port_registration_callback (jack_client_t *,
 					 registration_callback, void *arg);
 
 /**
- * Tell the Jack server to call 'registration_callback' whenever the processing
- * graph is reordered, passing 'arg' as an argument.
+ * Tell the JACK server to call @a graph_callback whenever the
+ * processing graph is reordered, passing @a arg as a parameter.
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_graph_order_callback (jack_client_t *, JackGraphOrderCallback graph_callback, void *);
+int jack_set_graph_order_callback (jack_client_t *,
+				   JackGraphOrderCallback graph_callback,
+				   void *);
 
 /**
- * Tell the Jack server to call 'xrun_callback' whenever there is a xrun, passing
- * 'arg' as an argument.
+ * Tell the JACK server to call @a xrun_callback whenever there is a
+ * xrun, passing @a arg as a parameter.
  *
  * @return 0 on success, otherwise a non-zero error code
  */
-int jack_set_xrun_callback (jack_client_t *, JackXRunCallback xrun_callback, void *arg);
+int jack_set_xrun_callback (jack_client_t *,
+			    JackXRunCallback xrun_callback, void *arg);
 
 /**
  * Tell the Jack server that the program is ready to start processing
@@ -489,6 +555,11 @@ jack_nframes_t jack_port_get_total_latency (jack_client_t *,
  * include both buffering by the audio interface *and* the converter. 
  */
 void jack_port_set_latency (jack_port_t *, jack_nframes_t);
+	
+/**
+ *
+ */
+int jack_recompute_total_latencies (jack_client_t*);
 
 /**
  * Modify a port's short name.  May be called at any time.  If the
@@ -643,7 +714,7 @@ jack_port_t *jack_port_by_name (jack_client_t *, const char *port_name);
 /**
  * @return address of the jack_port_t of a @a port_id.
  */
-jack_port_t *jack_port_by_id (const jack_client_t *client,
+jack_port_t *jack_port_by_id (jack_client_t *client,
 			      jack_port_id_t port_id);
 
 /**
@@ -680,7 +751,6 @@ jack_nframes_t jack_frame_time (const jack_client_t *);
  */
 jack_nframes_t jack_last_frame_time (const jack_client_t *client);
 
-
 /**
  * @return the current CPU load estimated by JACK.  This is a running
  * average of the time it takes to execute a full process cycle for
@@ -688,20 +758,7 @@ jack_nframes_t jack_last_frame_time (const jack_client_t *client);
  * determined by the buffer size and sample rate.
  */
 float jack_cpu_load (jack_client_t *client);
-
-/**
- * Set the directory in which the server is expected
- * to have put its communication FIFOs. A client
- * will need to call this before calling
- * jack_client_new() if the server was started
- * with arguments telling it to use a non-standard
- * directory.
- * 
- * @deprecated This function is deprecated.  Don't use in new programs
- * and remove it in old programs.
- */
-void jack_set_server_dir (const char *path);
-
+	
 /**
  * @return the pthread ID of the thread running the JACK client side
  * code.
