@@ -15,7 +15,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: port.h,v 1.19 2003/10/31 20:52:24 joq Exp $
+    $Id: port.h,v 1.20 2004/02/20 04:31:22 joq Exp $
 */
 
 #ifndef __jack_port_h__
@@ -26,7 +26,7 @@
 #include <jack/jslist.h>
 #include <jack/shm.h>
 
-#define JACK_PORT_NAME_SIZE 32
+#define JACK_PORT_NAME_SIZE 256
 #define JACK_PORT_TYPE_SIZE 32
 
 /* The relatively low value of this constant reflects the fact that
@@ -42,9 +42,7 @@
  */              
 #define JACK_MAX_PORT_TYPES 4
 #define JACK_AUDIO_PORT_TYPE 0
-
-/* these should probably go somewhere else, but not in <jack/types.h> */
-#define JACK_CLIENT_NAME_SIZE 32
+#define JACK_CLIENT_NAME_SIZE 33
 typedef uint32_t jack_client_id_t;
 
 /* JACK shared memory segments are limited to MAX_INT32, they can be
@@ -95,7 +93,7 @@ typedef struct _jack_port_shared {
     jack_shmsize_t           offset;	/* buffer offset in shm segment */
     jack_port_id_t           id;	/* index into engine port array */
     enum JackPortFlags	     flags;    
-    char                     name[JACK_CLIENT_NAME_SIZE+JACK_PORT_NAME_SIZE+2];
+    char                     name[JACK_CLIENT_NAME_SIZE+JACK_PORT_NAME_SIZE];
     jack_client_id_t         client_id;	/* who owns me */
 
     volatile jack_nframes_t  latency;
