@@ -1,5 +1,6 @@
 /*
     Copyright (C) 2001 Paul Davis
+    Copyright (C) 2004 Jack O'Quin
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -15,7 +16,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: types.h,v 1.22 2003/12/01 19:45:14 pbd Exp $
+    $Id: types.h,v 1.34 2004/12/25 18:39:44 joq Exp $
 */
 
 #ifndef __jack_types_h__
@@ -23,7 +24,6 @@
 
 #include <inttypes.h>
 
-typedef char    shm_name_t[32];
 typedef int32_t jack_shmsize_t;
 
 /**
@@ -41,15 +41,20 @@ typedef uint32_t	     jack_nframes_t;
  * Type used to represent the value of free running
  * monotonic clock with units of microseconds.
  */
-
-/* JOQ: this is trouble.  APPLE POWERPC should use a compatible
- * typedef, explicitly converting from double, if necessary.
- * Otherwise applications cannot safely print a jack_time_t. */
-#if defined(__APPLE__) && defined(__POWERPC__) 
-typedef double jack_time_t;
-#else
 typedef uint64_t jack_time_t;
-#endif
+
+/**
+ *  Maximum size of @a load_init string passed to an internal client
+ *  jack_initialize() function via jack_internal_client_load().
+ */
+#define JACK_LOAD_INIT_LIMIT 1024
+
+/**
+ *  jack_intclient_t is an opaque type representing a loaded internal
+ *  client.  You may only access it using the API provided in @ref
+ *  intclient.h "<jack/intclient.h>".
+ */
+typedef uint64_t jack_intclient_t;
 
 /**
  *  jack_port_t is an opaque type.  You may only access it using the
@@ -83,6 +88,20 @@ typedef uint32_t	     jack_port_id_t;
  */ 
 typedef int  (*JackProcessCallback)(jack_nframes_t nframes, void *arg);
 
+/**
+ * Prototype for the client supplied function that is called 
+ * once after the creation of the thread in which other
+ * callbacks will be made. Special thread characteristics
+ * can be set from this callback, for example. This is a
+ * highly specialized callback and most clients will not
+ * and should not use it.
+ *
+ * @param arg pointer to a client supplied structure
+ *
+ * @return void
+ */ 
+typedef void  (*JackThreadInitCallback)(void *arg);
+
 /**
  * Prototype for the client supplied function that is called 
  * whenever the processing graph is reordered.
@@ -94,8 +113,10 @@ typedef int  (*JackProcessCallback)(jack_nframes_t nframes, void *arg);
 typedef int  (*JackGraphOrderCallback)(void *arg);
 
 /**
- * Prototype for the client supplied function that is called 
- * whenever an xrun has occured.
+ * Prototype for the client-supplied function that is called whenever
+ * an xrun has occured.
+ *
+ * @see jack_get_xrun_delayed_usecs()
  *
  * @param arg pointer to a client supplied structure
  *
@@ -211,11 +232,141 @@ enum JackPortFlags {
       * for an output port: the data available at the port
       *                    does not originate from any other port
       *
-      * Audio synthesizers, i/o h/w interface clients, HDR
-      * systems are examples of things that would set this
-      * flag for their ports.  
+      * Audio synthesizers, I/O hardware interface clients, HDR
+      * systems are examples of clients that would set this flag for
+      * their ports.
       */
      JackPortIsTerminal = 0x10
 };	    
 
+/**
+ *  @ref jack_options_t bits
+ */
+enum JackOptions {
+
+     /**
+      * Null value to use when no option bits are needed.
+      */
+     JackNullOption = 0x00,
+
+     /**
+      * Do not automatically start the JACK server when it is not
+      * already running.  This option is always selected if
+      * \$JACK_NO_START_SERVER is defined in the calling process
+      * environment.
+      */
+     JackNoStartServer = 0x01,
+
+     /**
+      * Use the exact client name requested.  Otherwise, JACK
+      * automatically generates a unique one, if needed.
+      */
+     JackUseExactName = 0x02,
+
+     /**
+      * Open with optional <em>(char *) server_name</em> parameter.
+      */
+     JackServerName = 0x04,
+
+     /**
+      * Load internal client from optional <em>(char *)
+      * load_name</em>.  Otherwise use the @a client_name.
+      */
+     JackLoadName = 0x08,
+
+     /**
+      * Pass optional <em>(char *) load_init</em> string to the
+      * jack_initialize() entry point of an internal client.
+      */
+     JackLoadInit = 0x10
+};
+
+/** Valid options for opening an external client. */
+#define JackOpenOptions (JackServerName|JackNoStartServer|JackUseExactName)
+
+/** Valid options for loading an internal client. */
+#define JackLoadOptions (JackLoadInit|JackLoadName|JackUseExactName)
+
+/**
+ *  Options for several JACK operations, formed by OR-ing together the
+ *  relevant @ref JackOptions bits.
+ */
+typedef enum JackOptions jack_options_t;
+
+/**
+ *  @ref jack_status_t bits
+ */
+enum JackStatus {
+
+     /**
+      * Overall operation failed.
+      */
+     JackFailure = 0x01,
+
+     /**
+      * The operation contained an invalid or unsupported option.
+      */
+     JackInvalidOption = 0x02,
+
+     /**
+      * The desired client name was not unique.  With the @ref
+      * JackUseExactName option this situation is fatal.  Otherwise,
+      * the name was modified by appending a dash and a two-digit
+      * number in the range "-01" to "-99".  The
+      * jack_get_client_name() function will return the exact string
+      * that was used.  If the specified @a client_name plus these
+      * extra characters would be too long, the open fails instead.
+      */
+     JackNameNotUnique = 0x04,
+
+     /**
+      * The JACK server was started as a result of this operation.
+      * Otherwise, it was running already.  In either case the caller
+      * is now connected to jackd, so there is no race condition.
+      * When the server shuts down, the client will find out.
+      */
+     JackServerStarted = 0x08,
+
+     /**
+      * Unable to connect to the JACK server.
+      */
+     JackServerFailed = 0x10,
+
+     /**
+      * Communication error with the JACK server.
+      */
+     JackServerError = 0x20,
+
+     /**
+      * Requested client does not exist.
+      */
+     JackNoSuchClient = 0x40,
+
+     /**
+      * Unable to load internal client
+      */
+     JackLoadFailure = 0x80,
+
+     /**
+      * Unable to initialize client
+      */
+     JackInitFailure = 0x100,
+
+     /**
+      * Unable to access shared memory
+      */
+     JackShmFailure = 0x200,
+
+     /**
+      * Client's protocol version does not match
+      */
+     JackVersionError = 0x400
+};
+
+/**
+ *  Status word returned from several JACK operations, formed by
+ *  OR-ing together the relevant @ref JackStatus bits.
+ */
+typedef enum JackStatus jack_status_t;
+
 #endif /* __jack_types_h__ */
