@@ -17,7 +17,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: engine.c,v 1.125 2005/05/11 02:06:46 pbd Exp $
+    $Id: engine.c,v 1.126 2005/06/15 09:18:40 rncbc Exp $
 */
 
 #include <config.h>
@@ -657,7 +657,7 @@ jack_process_external(jack_engine_t *engine, JSList *node)
 	if (engine->freewheeling) {
 		poll_timeout = 10000; /* 10 seconds */
 	} else {
-		poll_timeout = (engine->control->real_time == 0 ?
+		poll_timeout = (engine->client_timeout_msecs > 0 ?
 				engine->client_timeout_msecs :
 				1 + engine->driver->period_usecs/1000);
 	}
