@@ -16,7 +16,7 @@
     along with this program; if not, write to the Free Software
     Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 
-    $Id: jackd.c,v 1.52 2005/01/16 01:56:45 joq Exp $
+    $Id: jackd.c,v 1.53 2005/06/15 09:18:40 rncbc Exp $
 */
 
 #include <config.h>
@@ -61,7 +61,7 @@ static int realtime_priority = 10;
 static int do_mlock = 1;
 static int temporary = 0;
 static int verbose = 0;
-static int client_timeout = 500; /* msecs */
+static int client_timeout = 0; /* msecs; if zero, use period size. */
 static unsigned int port_max = 128;
 static int do_unlock = 0;
 
@@ -135,7 +135,10 @@ jack_main (jack_driver_desc_t * driver_desc, JSList * driver_params)
 	 */
 
 	pthread_sigmask (SIG_BLOCK, &signals, 0);
-	
+
+	if (!realtime && client_timeout == 0)
+		client_timeout = 500; /* 0.5 sec; usable when non realtime. */
+
 	/* get the engine/driver started */
 
 	if ((engine = jack_engine_new (realtime, realtime_priority, 
