@@ -26,9 +26,9 @@
 #include <stdio.h>
 #include <jack/internal.h>
 #include <jack/engine.h>
+#include <jack/messagebuffer.h>
 #include "transengine.h"
 
-
 /********************** internal functions **********************/
 
 /* initiate polling a new slow-sync client
@@ -425,6 +425,7 @@ jack_transport_cycle_end (jack_engine_t *engine)
 		cmd = TransportCommandNone;
 
 	/* state transition switch */
+
 	switch (ectl->transport_state) {
 
 	case JackTransportStopped:
@@ -475,7 +476,7 @@ jack_transport_cycle_end (jack_engine_t *engine)
 			    ectl->transport_state);
 	}
 
-	/* update timebase, if needed */
+	/* Update timebase, if needed. */
 	if (ectl->transport_state == JackTransportRolling) {
 		ectl->pending_time.frame =
 			ectl->current_time.frame + ectl->buffer_size;
