@@ -1,6 +1,7 @@
 /* -*- mode: c; c-file-style: "bsd"; -*- */
 /*
     Copyright (C) 2001-2003 Paul Davis
+    Copyright (C) 2005 Jussi Laako
     
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU Lesser General Public License as published by
@@ -16,7 +17,7 @@
     along with this program; if not, write to the Free Software 
     Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 
-    $Id: client.c,v 1.121 2005/06/10 14:00:20 trutkin Exp $
+    $Id: client.c,v 1.125 2006/01/03 21:10:49 sonarnerd Exp $
 */
 
 #include <config.h>
@@ -46,6 +47,7 @@
 #include <jack/unlock.h>
 #include <jack/thread.h>
 #include <jack/varargs.h>
+#include <jack/intsimd.h>
 
 #include <sysdeps/time.h>
 JACK_TIME_GLOBAL_DECL;			/* One instance per process. */
@@ -66,18 +68,145 @@ JACK_TIME_GLOBAL_DECL;			/* One instance per process. */
 static pthread_mutex_t client_lock;
 static pthread_cond_t  client_ready;
 void *jack_zero_filled_buffer = NULL;
+#ifdef ARCH_X86
+int cpu_type = 0;
+#endif /* ARCH_X86 */
 
 #define EVENT_POLL_INDEX 0
 #define WAIT_POLL_INDEX 1
 #define event_fd pollfd[EVENT_POLL_INDEX].fd
 #define graph_wait_fd pollfd[WAIT_POLL_INDEX].fd
 
+
 typedef struct {
     int status;
     struct _jack_client *client;
     const char *client_name;
 } client_info;
 
+#ifdef USE_DYNSIMD
+
+#ifdef ARCH_X86
+
+static int
+have_3dnow ()
+{
+	unsigned int res = 0;
+
+#ifdef __x86_64__
+	asm volatile ("pushq %%rbx\n\t" : : : "memory");
+#else
+	asm volatile ("pushl %%ebx\n\t" : : : "memory");
+#endif
+	asm volatile (
+		"movl $0x80000000, %%eax\n\t" \
+		"cpuid\n\t" \
+		"cmpl $0x80000001, %%eax\n\t" \
+		"jl tdnow_prexit\n\t" \
+		\
+		"movl $0x80000001, %%eax\n\t" \
+		"cpuid\n\t" \
+		\
+		"xorl %%eax, %%eax\n\t" \
+		\
+		"movl $1, %%ecx\n\t" \
+		"shll $31, %%ecx\n\t" \
+		"testl %%ecx, %%edx\n\t" \
+		"jz tdnow_testexit\n\t" \
+		"movl $1, %%eax\n\t" \
+		\
+		"movl $1, %%ecx\n\t" \
+		"shll $30, %%ecx\n\t" \
+		"testl %%ecx, %%edx\n\t" \
+		"jz tdnow_testexit\n\t" \
+		"movl $2, %%eax\n\t" \
+		"jmp tdnow_testexit\n\t" \
+		\
+		"tdnow_prexit:\n\t" \
+		"xorl %%eax, %%eax\n\t" \
+		"tdnow_testexit:\n\t"
+		: "=a" (res)
+		:
+		: "ecx", "edx", "memory");
+#ifdef __x86_64__
+	asm volatile ("popq %%rbx\n\t" : : : "memory");
+#else
+	asm volatile ("popl %%ebx\n\t" : : : "memory");
+#endif
+	return res;
+}
+
+static int
+have_sse ()
+{
+	unsigned int res = 0;
+
+#ifdef __x86_64__
+	asm volatile ("pushq %%rbx\n\t" : : : "memory");
+#else
+	asm volatile ("pushl %%ebx\n\t" : : : "memory");
+#endif
+	asm volatile (
+		"movl $1, %%eax\n\t" \
+		"cpuid\n\t" \
+		\
+		"xorl %%eax, %%eax\n\t" \
+		\
+		"movl $1, %%ebx\n\t" \
+		"shll $25, %%ebx\n\t" \
+		"testl %%ebx, %%edx\n\t" \
+		"jz sse_testexit\n\t" \
+		"movl $1, %%eax\n\t" \
+		\
+		"movl $1, %%ebx\n\t" \
+		"shll $26, %%ebx\n\t" \
+		"testl %%ebx, %%edx\n\t" \
+		"jz sse_testexit\n\t" \
+		"movl $2, %%eax\n\t" \
+		\
+		"movl $1, %%ebx\n\t" \
+		"testl %%ebx, %%ecx\n\t" \
+		"jz sse_testexit\n\t" \
+		"movl $3, %%eax\n\t" \
+		\
+		"sse_testexit:\n\t"
+		: "=a" (res)
+		:
+		: "ecx", "edx", "memory");
+#ifdef __x86_64__
+	asm volatile ("popq %%rbx\n\t" : : : "memory");
+#else
+	asm volatile ("popl %%ebx\n\t" : : : "memory");
+#endif
+	return res;
+}
+
+static void
+init_cpu ()
+{
+	cpu_type = ((have_3dnow() << 8) | have_sse());
+	if (ARCH_X86_HAVE_3DNOW(cpu_type))
+		fprintf(stderr, "Enhanced3DNow! detected\n");
+	if (ARCH_X86_HAVE_SSE2(cpu_type))
+		fprintf(stderr, "SSE2 detected\n");
+	if ((!ARCH_X86_HAVE_3DNOW(cpu_type)) && (!ARCH_X86_HAVE_SSE2(cpu_type)))
+		fprintf(stderr,
+			"No supported SIMD instruction sets detected\n");
+	jack_port_set_funcs();
+}
+
+#else /* ARCH_X86 */
+
+static void
+init_cpu ()
+{
+	jack_port_set_funcs();
+}
+
+#endif /* ARCH_X86 */
+
+#endif /* USE_DYNSIMD */
+
 void 
 jack_error (const char *fmt, ...)
 {
@@ -199,6 +328,10 @@ jack_client_alloc ()
 	client->n_port_types = 0;
 	client->port_segment = NULL;
 
+#ifdef USE_DYNSIMD
+	init_cpu();
+#endif /* USE_DYNSIMD */
+
 	return client;
 }
 
