@@ -23,6 +23,7 @@ struct _jack_client {
     jack_shm_info_t*    port_segment;
 
     JSList *ports;
+    JSList *ports_ext;
 
     pthread_t thread;
     char fifo_prefix[PATH_MAX+1];
@@ -31,7 +32,8 @@ struct _jack_client {
     char thread_ok : 1;
     char first_active : 1;
     pthread_t thread_id;
-    
+    char name[JACK_CLIENT_NAME_SIZE];
+
 #ifdef JACK_USE_MACH_THREADS
     /* specific ressources for server/client real-time thread communication */
     mach_port_t clienttask, bp, serverport, replyport;
@@ -42,8 +44,11 @@ struct _jack_client {
 
 };
 
-extern int jack_client_deliver_request (const jack_client_t *client, jack_request_t *req);
-extern jack_port_t *jack_port_new (const jack_client_t *client, jack_port_id_t port_id, jack_control_t *control);
+extern int jack_client_deliver_request (const jack_client_t *client,
+					jack_request_t *req);
+extern jack_port_t *jack_port_new (const jack_client_t *client,
+				   jack_port_id_t port_id,
+				   jack_control_t *control);
 
 extern void *jack_zero_filled_buffer;
 
